import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

/**
 * Write a description of class world here.
 * 
 * @David Zhang
 * @Jan 21th, 2013
 */
public class Background extends World
{
    //This is where I declare all my instance variables and sounds for the game
    private ProHealth hpBar;
    private int count = 0;
    private int currentHP = 3;
    private ScoreKeeper scoreKeeper;
    private GreenfootSound theme = new GreenfootSound ("Main_Theme_From_Metal_Slug1.mp3");
    private GreenfootSound yell = new GreenfootSound ("Metal Slug.wav");
    private GreenfootSound startSound = new GreenfootSound ("Carry Out.wav");
    private GreenfootSound endSound = new GreenfootSound ("Allen 4.wav");
    private LifeCounter lifeCounter;
    private EndScreen endScreen;
    private int score = 0;
    //This is how many times the character has gotten hit 
    private int hitPoints = 0;
    private boolean yellCount = true;
    private int lives = 3;
    private boolean game = false;
    private PlayButton playButton = new PlayButton();
    private titlePage titlePage = new titlePage();

    /**
     * Constructor for objects of class world.
     * 
     */
    public Background()
    {    
        // Create a new world with 800x360 cells with a cell size of 1x1 pixels.
        super(800, 360, 1); 

        //These are all initializing some classes/objects and inputting the classes/objects into the world
        scoreKeeper = new ScoreKeeper();

        lifeCounter = new LifeCounter();

        addObject (scoreKeeper, 701, 18);
        addObject (lifeCounter, 71, 18); 

        addObject(new PaleMartian(), 762, 95);
        addObject(new PaleMartian(), 762, 155);
        addObject(new PaleMartian(), 762, 215);
        addObject(new PaleMartian(), 762, 275);
        addObject(new PaleMartian(), 762, 335);

        addObject(new Man(), 132, 180);
        //This inputs the values of score1 and lives into the world
        scoreKeeper.setScore(score);
        lifeCounter.setScore(lives);

        hpBar = new ProHealth (3);
        addObject (hpBar, 326, 25);

        addObject(titlePage, 400,180);
        addObject(playButton, 617, 255);
        playButton();
    }

    /**
     * Act - do whatever the man wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act()
    {
        startYell();
        gameMethods();
    }

    /**
     * This method makes the game say "Metal Slug" at the beginning and doesn't repeat ever again.
     */
    private void startYell()
    {
        //If yellCount is true, run the following code.
        if(yellCount)
        {
            //Play the yell sound
            yell.play();
            //Changes the yellCount boolean to false
            yellCount = false;
        }
    }

    /**
     * This method is purely to make the act method have as little code as possible.
     */
    private void gameMethods()
    {
        //If the boolean game is true than run the following code or else run the code underneath it.
        if(game)
        {
            //Run spawnNpc
            spawnNPC();  
            //If the counter is equal to 350, run the code which plays the theme music, this is because there is another sound file that is supposed to 
            // play before this one and that one last approx 350 acts.
            if(count ==350)
            {
                theme.playLoop();
            }
            //Only when count is smaller or equal to 349 does this following code runs, which is to increase the counter by one each act.
            if (count <=349)
            {
                count++;
            }

        } 
        //else run this part of the code.
        else
        {
            //The boolean game is only to see if you started the game or not, and if it isn't true run these lines of code, which puts the title page
            // onto the screen and runs the method playButton().
            addObject(titlePage, 400,180);
            addObject(playButton, 600, 200);
            playButton();
        }
    }

    /**
     * This is where when called, the scoreKeeper that I have in the world, adds a point so the points increase.
     */
    public void addPoint()
    {
        if (hitPoints != 9)
        {
            //This is the place where the score actually increases by 1 on top of itself
            score += 1;
            //Then it displays itself again (updates it self everytime it is called)
            scoreKeeper.setScore(score);
        }   
    }
    
    /**
     * This is where I intialize the use of the play button, letting the user play the game.
     */
    private void playButton()
    {
        //If the mose has clicked the play button, run the following code.
        if(Greenfoot.mouseClicked(playButton))
        {
            //First play the sound startSound
            startSound.play();
            //then remove the following, the title page and the play button which would systematically show the screen of the game
            removeObject(playButton);
            removeObject(titlePage);
            //then put the boolean game into true, signalling that the user has started the game.
            game = true;
        }
    }

    /**
     * This takes a life away when called and updates the lives screen/counter in the in-game-screen
     */
    public void subtractLife()
    {
        //If hit points is smaller than 8, put the currentHP back to 3, and update the hpBar.
        if (hitPoints <8)
        {
            currentHP = 3;
            hpBar.changeHP(currentHP);
        }
        //Then take away one life and update the lifeCounter
        lives -= 1;
        lifeCounter.setScore(lives);
    }

    /**
     * This gives a life back to the character when called and updates the screen
     */
    public void addLife()
    {
        //Add a life to lives, subtract hitPoints by 3, and updates the lifeCounter.
        lives++;
        hitPoints-=3;
        lifeCounter.setScore(lives);
    }

    /**
     * This takes away a health and updates the hpBar
     */
    public void subtractHealth()
    {
        //currentHP subtract one, update the hpBar, and increase the hitPoints by 1. 
        currentHP-=1;
        hpBar.changeHP(currentHP);
        hitPoints++;

    }

    /**
     * This method does the exact opposite of the method subtractHealth() above^
     */
    public void addHealth()
    {
        //CurrentHp add one, update the hpBar, and decrease the hitPoints by 1
        currentHP +=1;
        hpBar.changeHP(currentHP);
        hitPoints--;
    }

    /**
     * This method spawns the beggars if a randomNumber between 1 and 750 is 5 
     */
    private void spawnNPC()
    {
        //When hitPoints does not equal nine run this code
        if(hitPoints != 9)
        {
            //if randomNumber equals 5, spawn a beggar at a random locations within the given parameters
            if (Greenfoot.getRandomNumber(750) == 5)
            {
                addObject (new Beggar(), Greenfoot.getRandomNumber(724), Greenfoot.getRandomNumber(285)+ 70);
            }
        }
    }

    /**
     * This method runs when the game is over
     */
    public void gameOver()
    {
        //Set the theme volume to 0, therefore muting it
        theme.setVolume(0);
        //play the endSound
        endSound.play();
        //get all the objects in the world and remove all them
        List objects = getObjects(null);
        removeObjects(objects);
        //get a new object called endScreen and add that into the world
        endScreen = new EndScreen();
        endScreen.setScore(score);
        addObject(endScreen, 400, 180);
    }
    
    /**
     * This method is so that when the game is running, it returns a boolean called game. 
     */
    public boolean gameRunning()
    {
        return game;
    }
}
